package com.anyway.favor.service.impl;

import com.anyway.favor.model.dto.FavorDto;
import com.anyway.favor.model.dto.FavorItemDto;
import com.anyway.favor.dao.FavorDao;
import com.anyway.favor.dao.FavorItemDao;
import com.anyway.favor.dao.GiftDao;
import com.anyway.favor.model.Favor;
import com.anyway.favor.model.FavorItem;
import com.anyway.favor.model.Gift;
import com.anyway.favor.service.FavorService;
import com.anyway.util.PageQuery;
import com.anyway.util.PageUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 人情业务实现类
 *
 * @author liuchuanwei
 * @date 2024-02-24
 */
@Slf4j
@Service
public class FavorServiceImpl implements FavorService {
    @Autowired
    private FavorDao favorDao;
    @Autowired
    private FavorItemDao favorItemDao;
    @Autowired
    private GiftDao giftDao;

    @Override
    public List<Favor> findByCondition(Map<String, Object> map) {
        return favorDao.findByCondition(map);
    }

    @Override
    public List<Favor> findPage(PageQuery<Map<String, Object>> pageQuery) {
        PageUtils.startPage(pageQuery);
        List<Favor> favorList = favorDao.findByCondition(pageQuery.getTerms());
        PageUtils.setPageTotal(favorList, pageQuery);
        return favorList;
    }

    @Transactional
    @Override
    public boolean add(FavorDto favorAddDto) {
        Favor favor = favorAddDto.toFavor();
        int r = favorDao.add(favor);
        if (r <= 0) {
            log.error("新增人情事件失败：{}", favorAddDto);
            return false;
        }
        List<FavorItem> favorItemList = new ArrayList<>();
        for (FavorItemDto favorItemAddDto : favorAddDto.getFavorItemList()) {
            FavorItem favorItem = favorItemAddDto.toFavorItem();
            favorItem.setFavorId(favor.getId());
            favorItem.setCreateBy(favor.getCreateBy());
            if (favorItemAddDto.getGiveGift() != null) {
                Gift giveGift = favorItemAddDto.getGiveGift();
                giftDao.add(giveGift);
                favorItem.setGiveGiftId(giveGift.getId());
            }
            if (favorItemAddDto.getReturnGift() != null) {
                Gift returnGift = favorItemAddDto.getReturnGift();
                giftDao.add(returnGift);
                favorItem.setReturnGiftId(returnGift.getId());
            }
            favorItemList.add(favorItem);
        }
        favorItemDao.batchAdd(favorItemList);
        return true;
    }

    @Transactional
    @Override
    public boolean update(FavorDto favorAddDto) {
        Favor favor = favorAddDto.toFavor();
        int r = favorDao.update(favor);
        if (r <= 0) {
            log.error("更新人情事件失败：{}", favorAddDto);
            return false;
        }

        //查询人情事件的所有明细，筛选出删除的
        List<FavorItemDto> allFavorItemList = favorItemDao.findByFavorId(favor.getId());
        Map<Long, FavorItem> favorItemMap = allFavorItemList.stream().collect(Collectors.toMap(FavorItem::getId, Function.identity(), (v1, v2) -> v1));
        //需要新增的人情明细
        List<FavorItem> newFavorItemList = new ArrayList<>();
        List<FavorItemDto> favorItemDtoList = favorAddDto.getFavorItemList();
        for (FavorItemDto favorItemAddDto : favorItemDtoList) {
            FavorItem favorItem = favorItemAddDto.toFavorItem();
            if (favorItemAddDto.getGiveGift() != null) {
                Gift giveGift = favorItemAddDto.getGiveGift();
                this.saveGift(giveGift);
                favorItem.setGiveGiftId(giveGift.getId());
            }
            if (favorItemAddDto.getReturnGift() != null) {
                Gift returnGift = favorItemAddDto.getReturnGift();
                this.saveGift(returnGift);
                favorItem.setReturnGiftId(returnGift.getId());
            }
            //更新
            if(favorItem.getId() != null) {
                favorItemMap.remove(favorItem.getId());
                favorItem.setModifyBy(favor.getModifyBy());
                favorItemDao.update(favorItem);
            } else { // 新增
                favorItem.setFavorId(favor.getId());
                favorItem.setCreateBy(favor.getModifyBy());
                newFavorItemList.add(favorItem);
            }
        }
        //新增人情明细
        if (!CollectionUtils.isEmpty(newFavorItemList)) {
            favorItemDao.batchAdd(newFavorItemList);
        }
        //删除人情明细以及礼物
        if (!CollectionUtils.isEmpty(favorItemMap)) {
            Set<Long> deleteFavorItemIdSet = favorItemMap.keySet();
            Set<Long> deleteGiftIdSet = new HashSet<>();
            for (Long deleteKey : deleteFavorItemIdSet) {
                FavorItem favorItem = favorItemMap.get(deleteKey);
                deleteGiftIdSet.add(favorItem.getGiveGiftId());
                if (favorItem.getReturnGiftId() != null) {
                    deleteGiftIdSet.add(favorItem.getReturnGiftId());
                }
            }
            //删除
            favorItemDao.deleteByIds(deleteFavorItemIdSet);
            giftDao.deleteByIds(deleteGiftIdSet);
        }
        return true;
    }

    /**
     * 保存礼物（新增或更新）
     * @param gift
     * @return
     */
    private int saveGift(Gift gift) {
        if (gift.getId() != null) {
            return giftDao.update(gift);
        } else {
            return giftDao.add(gift);
        }
    }

    @Override
    public FavorDto findDetail(Long id) {
        Favor favor = favorDao.findById(id);
        List<FavorItemDto> favorItemList = favorItemDao.findByFavorId(favor.getId());

        FavorDto favorDto = new FavorDto(favor);
        favorDto.setFavorItemList(favorItemList);

        return favorDto;
    }

    @Override
    public boolean deleteById(Long id) {
        int i = favorDao.deleteById(id);
        if (i>0) {
            List<FavorItemDto> favorItemList = favorItemDao.findByFavorId(id);
            Set<Long> giftIdSet = new HashSet<>();
            for (FavorItem favorItem : favorItemList) {
                giftIdSet.add(favorItem.getGiveGiftId());
                if (favorItem.getReturnGiftId() != null) {
                    giftIdSet.add(favorItem.getReturnGiftId());
                }
            }
            giftDao.deleteByIds(giftIdSet);
            favorItemDao.deleteByFavorId(id);
        }
        return i>0;
    }
}
